package de.moritzruth.dracula_musical.device

import de.moritzruth.theaterdsl.device.*
import de.moritzruth.theaterdsl.dmx.DmxAddress
import de.moritzruth.theaterdsl.dmx.DmxDataWriter
import de.moritzruth.theaterdsl.dmx.DmxValue
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.percent
import kotlinx.collections.immutable.ImmutableSet
import kotlinx.collections.immutable.persistentSetOf
import kotlin.math.roundToInt

class StairvilleSplb(override val firstChannel: DmxAddress) : Device {
    override val numberOfChannels: UInt = 11u

    val brightness = PercentageDV()
    val strobeFrequency = PercentageDV()

    val color = ColorDV(Color.WHITE.copy(brightness = 0.percent))
    val white = PercentageDV()

    val effect = ConcreteDV<Effect?>(null)
    val effectSpeed = PercentageDV()

    enum class Effect {
        HECTIC_SWITCHING,
        THEATRE_SWITCHING,
        CHASE_BACK_AND_FORTH,
        LAZY_SPARKLES,
        VOLATILE_SPARKLES,
        BUMP_INWARDS,
        FLOW_INWARDS,
        FLOW_INWARDS_LONG,
        FLOW_OUTWARDS,
        FLOW_OUTWARDS_LONG;
    }

    override val dvs: ImmutableSet<DynamicValue<*>> = persistentSetOf(
        brightness,
        strobeFrequency,
        color,
        white,
        effect,
        effectSpeed
    )

    override fun writeDmxData(writer: DmxDataWriter, isLightBehindCurtainOn: Boolean) {
        writer.writePercentage(brightness.getCurrentValue())

        val strobeFrequency = strobeFrequency.getCurrentValue()
        if (strobeFrequency == 0.percent) writer.writeRaw(DmxValue(0u))
        else writer.writeRaw(DmxValue(strobeFrequency.ofRange(11.0..255.0).roundToInt().toUByte()))

        // mode
        val preset = effect.getCurrentValue()
        if (preset == null) writer.writeRaw(DmxValue(0u)) // mode = RGBW
        else writer.writeRaw(DmxValue(128u)) // mode = preset

        writer.writeRaw(DmxValue(0u)) // color preset (not used)

        if (preset == null) writer.writeRaw(DmxValue(0u))
        else writer.writeRaw(DmxValue(0u))

        writer.writeRaw(DmxValue(0u)) // sound mode preset

        writer.writePercentage(effectSpeed.getCurrentValue())

        val (red, green, blue) = color.getCurrentValue().getRGB()
        writer.writePercentage(red)
        writer.writePercentage(green)
        writer.writePercentage(blue)
        writer.writePercentage(white.getCurrentValue())
    }
}