package de.moritzruth.dracula_musical.song

import de.moritzruth.dracula_musical.device.*
import de.moritzruth.theaterdsl.show.*
import de.moritzruth.theaterdsl.value.Color
import de.moritzruth.theaterdsl.value.degrees
import de.moritzruth.theaterdsl.value.percent
import kotlin.time.Duration.Companion.minutes
import kotlin.time.Duration.Companion.seconds

private fun StairvilleTlb.breathe() = brightness.sine(1.seconds, 20.percent, 100.percent)
private fun StairvilleTlb.pulseSlow() = brightness.pulse(0.8.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 100.percent)
private fun StairvilleTlb.pulseFast() = brightness.pulse(0.6.seconds, 0.1.seconds, 0.2.seconds, startDelay = 0.seconds, peak = 100.percent)

fun SceneBuilderContext.songFinaleErsterAkt() {
    step(StepCue.MusicStart("Finale (1. Akt)", 4.minutes + 40.seconds)) {
        onRun {
            Washs.both {
                it.pointAtAudience()
                it.colorWheelMode.static(CoemarProWash.ColorWheelMode.WarmWhite)
                it.beamAngle.static(100.percent)
            }
        }
    }

    step(StepCue.Text("Dracula", "Guten Abend, Mister Harker.")) {
        actors {
            -"Jonathan / nach rechts, mit Vertrag"
        }

        onRun {
            FrontLights.left { it.brightness.off(10.seconds) }
            FrontLights.all { it.brightness.fade(30.percent, 10.seconds) }
            FrontLights.right { it.brightness.off(10.seconds) }

            Spots.right.brightness.fade(100.percent, 10.seconds)

            backlightBar.brightness.fade(100.percent, 10.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "Und du lässt ihn geh’n!", "letzte Silbe")) {
        sidelight.brightness.static(100.percent)
        backlightBar.breathe()

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.VOLATILE_SPARKLES)
            it.effect.speed.static(50.percent)
            it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, end = 25.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "…ganz allein du!", "letzte Silbe")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.BUMP_INWARDS)
            it.effect.speed.static(50.percent)
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds, end = 25.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.WHITE, 0.1.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.CHASE_BACK_AND_FORTH)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "letzte Silbe")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
            it.effect.static(null)
            it.color.static(Color.RED)
        }
    }

    lightStep(StepCue.Text("Dracula", "Ich will diese Frau!", "Anfang")) {
        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.BUMP_INWARDS)
            it.effect.speed.static(50.percent)
            it.brightness.fade(25.percent, 0.4.seconds)
        }
    }

    lightStep(StepCue.Text("Françoise", "Und was ist mit uns?", "Anfang")) {
        backlightBar.breathe()
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.CHASE_BACK_AND_FORTH)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "letzte Silbe")) {
        backlightBar.breathe()
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
            it.effect.static(null)
            it.color.static(Color.RED)
        }
    }

    greedyCueStep(StepCue.Text("Vampire", "Wir sollen wohl gar nichts haben!", "Anfang"))

    lightStep(StepCue.Text("Vampire", "Uns’re Kehlen sind verdorrt!", "Anfang")) {
        backlightBar.pulseSlow()
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.CHASE_BACK_AND_FORTH)
            it.effect.speed.static(50.percent)
            it.brightness.fade(25.percent, 0.4.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "…machen nicht satt!", "letzte Silbe")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
            it.effect.static(null)
            it.color.static(Color.RED)
        }
    }

    lightStep(StepCue.Text("Dracula", "Ich sorg für euch!", "Anfang")) {
        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.BUMP_INWARDS)
            it.effect.speed.static(50.percent)
            it.brightness.fade(25.percent, 0.4.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.WHITE, 0.1.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.CHASE_BACK_AND_FORTH)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "letzte Silbe")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
            it.effect.static(null)
            it.color.static(Color.RED)
        }
    }

    lightStep(StepCue.Text("Dracula", "Ihr müsst mir vertrau’n…", "Anfang")) {
        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.BUMP_INWARDS)
            it.effect.speed.static(50.percent)
            it.brightness.fade(25.percent, 0.4.seconds)
        }
    }

    greedyCueStep(StepCue.Text("Dracula", "…habt doch Geduld!", "Anfang"))

    lightStep(StepCue.Text("Françoise", "Du sorgst für uns?", "Anfang")) {
        backlightBar.breathe()
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.CHASE_BACK_AND_FORTH)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen Blut!", "letzte Silbe")) {
        backlightBar.breathe()
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
            it.effect.static(null)
            it.color.static(Color.RED)
        }
    }

    lightStep(StepCue.Text("Bernadette", "Das Essen ist weg…", "Anfang")) {
        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.VOLATILE_SPARKLES)
            it.effect.speed.static(50.percent)
            it.brightness.fade(25.percent, 0.4.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "…und du bist Schuld!", "letzte Silbe")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
            it.effect.static(null)
            it.color.static(Color.RED)
        }
    }

    lightStep(StepCue.Text("Dracula", "Ich brauche ihn noch!", "Anfang")) {
        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.FLOW_INWARDS)
            it.effect.speed.static(50.percent)
            it.brightness.fade(25.percent, 0.4.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "Die Frau ist weg…", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.HECTIC_SWITCHING)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Dracula", "Ich will sie für euch!", "Anfang")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.FLOW_INWARDS)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir hatten den Mann…", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.HECTIC_SWITCHING)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Dracula", "Sie ist wunderschön!", "Anfang")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.FLOW_INWARDS)
            it.effect.speed.static(50.percent)
        }
    }

    greedyCueStep(StepCue.Text("Dracula", "Viel schöner als er!", "Anfang"))

    lightStep(StepCue.Text("Vampire", "Das ist uns egal!", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.HECTIC_SWITCHING)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Dracula", "So schaut sie doch an!", "Anfang")) {
        backlightBar.brightness.fade(20.percent, 0.4.seconds)
        backlightBar.color.fade(Color.RED, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.FLOW_INWARDS)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wir wollen ihr Blut…", "Anfang")) {
        backlightBar.brightness.fade(100.percent, 0.4.seconds)
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.HECTIC_SWITCHING)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Wenn man…", "Anfang")) {
        backlightBar.breathe()
        BlinderBars.all { it.brightness.pulseOnce(5.seconds, 1.seconds) }
    }

    lightStep(StepCue.Text("Dracula", "Die Frau!", "Anfang")) {
        backlightBar.brightness.fade(20.percent, 0.2.seconds)
        backlightBar.color.fade(Color.RED, 0.2.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.FLOW_INWARDS)
            it.effect.speed.static(70.percent)
        }
    }

    lightStep(StepCue.Text("Vampire", "Nein!", "Anfang")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 0.3.seconds, end = 20.percent)
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.effect.static(null)
            it.color.static(Color.RED)
            it.brightness.pulseOnce(0.1.seconds, 0.3.seconds)
        }
    }

    lightStep(StepCue.Text("Dracula", "Sie ist so schön!", "Anfang")) {
        backlightBar.brightness.fade(20.percent, 0.1.seconds)
        backlightBar.color.fade(Color.RED, 0.1.seconds)
    }

    lightStep(StepCue.Text("Dracula", "Sie ist so schön!", "letzte Silbe")) {
        BlinderBars.all {
            it.color.reset()
            it.white.static(100.percent)
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "Nein!", "Anfang")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 0.3.seconds, end = 20.percent)
        backlightBar.color.fade(Color.WHITE, 0.4.seconds)

        BlinderBars.all {
            it.color.static(Color.RED)
            it.white.reset()
            it.brightness.pulseOnce(0.1.seconds, 0.2.seconds)
        }
    }

    greedyCueStep(StepCue.Text("Vampire", "Das ist egal!", "Anfang"))

    lightStep(StepCue.Text("Vampire", "Das ist egal!", "letzte Silbe")) {
        sidelight.brightness.fade(start = 0.percent, end = 100.percent, duration = 0.4.seconds)
        sidelight.strobeSpeed.off()
    }

    greedyCueStep(StepCue.Text("Dracula", "Schaut sie wenigstens an!", "Anfang"))

    lightStep(StepCue.Text("Dracula", "Schaut sie wenigstens an!", "letzte Silbe")) {
        backlightBar.color.fade(Color.WARM_WHITE, 6.seconds)
        backlightBar.brightness.fade(60.percent, 10.seconds)

        sidelight.colors.forEach { it.fade(Color.WARM_WHITE, 10.seconds) }

        Washs.both {
            it.pointAtCeiling(10.seconds)
            it.brightness.fade(100.percent, 3.seconds)
        }

        Tops.both {
            it.startRoomMovement(3.0)
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.Rotate(10.percent))
            it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
        }

        FrontLights.center { it.brightness.fade(100.percent, 6.seconds) }
    }

    // genre change

    lightStep(StepCue.Custom("Musik: Fill-in")) {
        Washs.both { it.brightness.off(5.seconds) }
        Tops.both { it.brightness.fade(100.percent, 5.seconds) }

        BlinderBars.all {
            it.color.reset()
            it.effect.static(StairvilleSplb.Effect.FLOW_INWARDS)
            it.effect.speed.static(15.percent)
            it.brightness.fade(30.percent, 2.seconds)
        }
    }

    lightStep(StepCue.Text("Vampire", "Oh ja", "Anfang")) {
        FrontLights.left { it.brightness.pulseOnce(2.seconds, 4.seconds, peak = 50.percent) }
        FrontLights.right { it.brightness.pulseOnce(2.seconds, 4.seconds, peak = 50.percent) }
    }

    lightStep(StepCue.Text("Vampire", "Oh ja", "Anfang")) {
        FrontLights.left { it.brightness.pulseOnce(2.seconds, 5.seconds, peak = 50.percent) }
        FrontLights.right { it.brightness.pulseOnce(2.seconds, 5.seconds, peak = 50.percent) }
        Washs.both { it.brightness.pulseOnce(2.seconds, 5.seconds) }
    }

    lightStep(StepCue.Text("Dracula", "…morgen nacht", "Anfang")) {
        BlinderBars.all { it.brightness.off(2.seconds) }
    }

    // genre change

    lightStep(StepCue.Text("Dracula", "…morgen nacht", "letzte Silbe")) {
        BlinderBars.all {
            it.brightness.pulseOnce(0.4.seconds, 0.4.seconds, peak = 60.percent, end = 30.percent)
            it.effect.static(StairvilleSplb.Effect.THEATRE_SWITCHING)
            it.effect.speed.static(40.percent)
        }
    }

    lightStep(StepCue.Text("Alle", "Komm zu uns morgen nacht!", "letzte Silbe")) {
        FrontLights.center { it.brightness.fade(30.percent, 4.seconds) }
        BlinderBars.all { it.brightness.off(2.seconds) }
        sidelight.brightness.off(2.seconds)
        Tops.both { it.brightness.off(2.seconds) }
    }

    // genre change

    lightStep(StepCue.Custom("Musik: Fill-in")) {
        backlightBar.color.fade(Color.WHITE, 1.seconds)
        backlightBar.breathe()

        sidelight.color1.fadeRandomAround(230.degrees, 30.degrees, 2.seconds)
        sidelight.color2.fadeRandomAround(230.degrees, 35.degrees, 4.seconds)
        sidelight.color3.static(Color(hue = 230.degrees, saturation = 20.percent))
        sidelight.color4.static(Color(hue = 230.degrees, saturation = 20.percent))
        sidelight.brightness.fade(100.percent, 1.seconds)

        BlinderBars.all {
            it.brightness.fade(25.percent, 1.seconds)
            it.effect.static(StairvilleSplb.Effect.BUMP_INWARDS)
            it.effect.speed.static(50.percent)
        }
    }

    lightStep(StepCue.Text("Dracula", "Mina…", "Anfang")) {
        backlightBar.color.fade(Color.WARM_WHITE, 3.seconds)
        backlightBar.brightness.fade(50.percent, 3.seconds)
    }

    lightStep(StepCue.Text("Vampire", "Komm zu uns morgen nacht!", "Anfang")) {
        FrontLights.center { it.brightness.fade(100.percent, 5.seconds) }
        BlinderBars.all { it.effect.speed.fade(1.percent, 3.seconds) }
        sidelight.colors.forEach { it.fade(Color.WARM_WHITE, 8.seconds) }
    }

    lightStep(StepCue.Text("Vampire", "Komm zu uns morgen nacht!", "letzte Silbe")) {
        BlinderBars.all { it.brightness.off(2.seconds) }
    }

    // genre change

    lightStep(StepCue.Text("Dracula", "Du musst ein wenig sterben…", "Anfang")) {
        BlinderBars.all {
            it.brightness.fade(30.percent, 1.seconds)
            it.effect.static(StairvilleSplb.Effect.THEATRE_SWITCHING)
            it.effect.speed.static(40.percent)
        }
    }

    lightStep(StepCue.Text("Dracula", "Mina…")) {
        backlightBar.brightness.off(5.seconds)
    }

    lightStep(StepCue.Custom("Musik: Schlag nach »Morgen!«")) {
        BlinderBars.all {
            it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, end = 30.percent)
            it.effect.speed.fade(0.percent, 0.4.seconds)
        }
    }

    lightStep(StepCue.Custom("Musik: Offbeat nach »Morgen!«")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds, end = 30.percent) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds) }
    }

    lightStep(StepCue.Text("Dracula", "Morgen nacht!", "letzte Silbe")) {
        BlinderBars.all { it.effect.speed.fade(40.percent, 1.seconds) }
    }

    lightStep(StepCue.Custom("Musik: 1. Beat nach »Morgen!«")) {
        backlightBar.brightness.pulseOnce(0.1.seconds, 0.5.seconds)
    }

    lightStep(StepCue.Custom("Musik: 2. Beat nach »Morgen!«")) {
        BlinderBars.all { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds) }
        Washs.both { it.brightness.pulseOnce(0.1.seconds, 0.5.seconds) }
    }

    lightStep(StepCue.Text("Vampire", "Morgen nacht!", "letzte Silbe")) {
        backlightBar.pulseFast()
        backlightBar.color.fade(Color.RED, 6.seconds)

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.BUMP_INWARDS)
            it.effect.speed.static(60.percent)
            it.brightness.fade(40.percent, 3.seconds)
        }
    }

    lightStep(StepCue.Text("Dracula", "Mina…")) {
        backlightBar.brightness.off(0.5.seconds)
        sidelight.brightness.fade(30.percent, 5.seconds)
        BlinderBars.all { it.brightness.off(1.seconds) }
    }

    lightStep(StepCue.Text("Dracula", "Morgen nacht!", "letzte Silbe")) {
        backlightBar.breathe()
        backlightBar.strobeSpeed.fade(start = 50.percent, end = 90.percent, duration = 3.5.seconds)

        sidelight.brightness.static(100.percent)
        sidelight.strobeSpeed.fade(start = 50.percent, end = 90.percent, duration = 3.5.seconds)

        Washs.both { it.brightness.rampDown(0.8.seconds) }

        BlinderBars.all {
            it.effect.static(StairvilleSplb.Effect.HECTIC_SWITCHING)
            it.effect.speed.static(80.percent)
            it.brightness.fade(60.percent, 5.seconds)
        }

        FrontLights.center { it.brightness.fade(100.percent, 4.seconds) }

        Tops.both {
            it.pointAtStageMiddleSingleCenterMarking()
            it.colorWheelMode.static(FuturelightDmh160.ColorWheelMode.White)
            it.prismMode.static(FuturelightDmh160.PrismMode.FROST)
        }
    }

    lightStep(StepCue.Custom("Musik: Schlussschlag")) {
        Spots.right.brightness.off()
        FrontLights.center { it.brightness.fade(start = 40.percent, end = 0.percent, duration = 10.seconds) }
        Washs.both { it.brightness.off(10.seconds) }
        Tops.both { it.brightness.pulseOnce(0.2.seconds, 10.seconds) }

        backlightBar.strobeSpeed.off()
        backlightBar.brightness.pulseOnce(0.2.seconds, 10.seconds)

        sidelight.strobeSpeed.off()
        sidelight.brightness.pulseOnce(0.2.seconds, 10.seconds)

        BlinderBars.all {
            it.effect.static(null)
            it.color.static(Color.WHITE)
            it.white.static(100.percent)
            it.brightness.pulseOnce(0.2.seconds, 3.seconds)
        }
    }

    step(StepCue.MusicEnd) {
        actors {
            -"Backe"
            -"Timmy"
            -"Eloïse"
            -"Françoise"
            -"Bernadette"
            -"Dracula"
            -"Vampire"
        }

        props {
            it[PropPosition.LEFT] = null
            it[PropPosition.BACKDROP] = "Dorfplatz"
        }

        curtainState = CurtainState.CLOSED

        rightSpotTarget = null
    }
}