package de.moritzruth.theaterdsl.device

import de.moritzruth.theaterdsl.value.percent
import kotlin.math.roundToLong
import kotlin.time.TimeMark
import kotlin.time.TimeSource

abstract class EffectDV<T, R>(private val initialEffect: T): DynamicValue<R> {
    private var time = 0L
    private var effect: T = initialEffect
    private var lastProgressMark: TimeMark? = null
    val speed = PercentageDV(100.percent)

    fun static(effect: T) {
        this.effect = effect
    }

    override fun reset() {
        time = 0
        effect = initialEffect
        lastProgressMark = null
        speed.reset()
    }

    override fun skipTransition() {
        speed.skipTransition()
    }

    override fun getCurrentValue(): R {
        val delta = ((lastProgressMark?.elapsedNow()?.inWholeMilliseconds ?: 0) * speed.getCurrentValue().value).roundToLong()
        time += delta
        lastProgressMark = TimeSource.Monotonic.markNow()

        return compute(time, effect)
    }

    abstract fun compute(time: Long, effect: T?): R
}